/* Include files.
 */
#include "XREA.h"				/* Standard XREA header file. */
#include "NSIDC_Reader.h"


/* Constants.
 */
#define NIL		0L


/* Prototypes.
 */
pascal Boolean	_main(int, Ptr, Ptr);
Boolean			FileFilter(ParmBlkPtr);
Boolean			FileRead(XREAInData*, XREAOutData*);


pascal Boolean
main(message, inData, outData)
	int		message;
	Ptr		inData;
	Ptr		outData;
{

	switch(message) {
		case FILE_FILTER:
			return(FileFilter((ParmBlkPtr)inData));
			break;
		case FILE_READ:
			return(FileRead((XREAInData*)inData, (XREAOutData*)outData));
			break;
		default:
			/* Unknown message.  Do nothing. */
			return(FALSE);
			break;
	}
}


Boolean
FileFilter(data)
	ParmBlkPtr	data;
{
	/* Show only TEXT files.
	 */
	if(data->fileParam.ioFlFndrInfo.fdType == 'TEXT') return(FALSE);
	else	return(TRUE);
}


Boolean
FileRead(inData, outData)
	XREAInData	*inData;
	XREAOutData	*outData;
{
	int				filetype;
	long			fileSize;
	OSErr			error;
	int				channel;
	int				nChannels;
	long			allocSize;
	Handle			imgData;
	unsigned short	*dataLine, *linePtr;
	unsigned char	*imgPtr;
	long			lineLength;
	long			count;
	int				xdim, ydim;
	int				i, j;
	unsigned short	value;
	char			string[10];
	DialogPtr		loadingDialog;

	/* Get the size of the input file.  This tells us which type of file
	 * we are dealing with.
	 */
	error = GetEOF(inData->refNum, &fileSize);
	if(error != noErr)	return(FALSE);

	/* Figure out which type of file this is.
	 */
	switch(fileSize) {
		case 2179072:		/* Size of N3A files. */
			filetype = N3A;
			xdim = 608;
			ydim = 896;
			nChannels = 2;
			break;
		case 1678592:		/* Size of S3A files. */
			filetype = S3A;
			xdim = 632;
			ydim = 664;
			nChannels = 2;
			break;
		case 1361920:		/* Size of N3B files. */
			filetype = N3B;
			xdim = 304;
			ydim = 448;
			nChannels = 5;
			break;
		case 1049120:		/* Size of S3B files. */
			filetype = S3B;
			xdim = 316;
			ydim = 332;
			nChannels = 5;
			break;
		case 544768:		/* N3A overlay. */
			filetype = N3A_O;
			xdim = 608;
			ydim = 896;
			break;
		case 419648:		/* S3A overlay. */
			filetype = N3B_O;
			xdim = 632;
			ydim = 664;
			break;
		case 136192:		/* N3B overlay. */
			filetype = S3A_O;
			xdim = 304;
			ydim = 448;
			break;
		case 104912:		/* S3B overlay. */
			filetype = S3B_O;
			xdim = 316;
			ydim = 332;
			break;
		default:
			Alert(BAD_FILE_DIALOG, NIL);
			return(FALSE);
	}

	/* Get which channel to extract.
	 */
	channel = 0;
	if(filetype == N3A || filetype == S3A) {
		channel = Alert(TYPE_A_DIALOG, NIL);
	}
	else if(filetype == N3B || filetype == S3B) {
		channel = Alert(TYPE_B_DIALOG, NIL);
	}
	if(channel == CANCEL)	return(FALSE);
	channel--;

	/* Allocate storage for the image data.
	 */
	allocSize = (long)xdim * (long)ydim;
	imgData = NewHandle(allocSize);
	if(imgData == NIL)	return(FALSE);
	MoveHHi(imgData);
	HLock(imgData);
	imgPtr = (unsigned char*)(*imgData);

	/* Show watch cursor and put up Loading dialog.
	 */
	SetCursor(*GetCursor(watchCursor));
	loadingDialog = GetNewDialog(LOADING_DIALOG, NIL, (WindowPtr)-1L);
	DrawDialog(loadingDialog);

	/* If we are reading one of the multispectral images, allocate storage for
	 * one line of input data and then proceed with the conversion.
	 */
	if(filetype >= N3A && filetype <= S3B) {

		/* Allocate temporary storage.
		 */
		lineLength = (long)nChannels * 2L * (long)xdim;
		dataLine = (unsigned short*)NewPtr(lineLength);
		if(dataLine == NIL) {
			DisposHandle(imgData);
			return(FALSE);
		}

		/* Conversion Loop.
		 */
		for(j=0; j<ydim; j++) {

			/* Read a line's worth of data.
			 */
			count = lineLength;
			error = FSRead(inData->refNum, &count, dataLine);
			if(error != noErr) {
	/*			+++ Report the error reading.*/
				DisposHandle(imgData);
				DisposPtr(dataLine);
				return(FALSE);
			}
			linePtr = dataLine;

			/* Skip any channels which may come before the one we're after.
			 */
			linePtr += (channel-1);

			/* Convert the data line.
			 */
			for(i=0; i<xdim; i++) {
				value = *linePtr & 0xFF00;
				value >>= 8;
				value &= 0x00FF;
				value += *linePtr << 8;
				*imgPtr = (unsigned char)((double)(value-1500) / 2000.0 * 256.0);
				imgPtr++;
				linePtr += nChannels;
			}
		}

		/* Dispose of the temporary storage.
		 */
		DisposPtr(dataLine);
	}
	else {
		error = FSRead(inData->refNum, &fileSize, imgPtr);
		if(error != noErr) {
/*			+++ Report the error reading.*/
			DisposHandle(imgData);
			return(FALSE);
		}
	}

	/* Show arrow cursor and put away Loading dialog.
	 */
	SetCursor(&arrow);
	DisposDialog(loadingDialog);

	/* Fill up the output data structure.
	 */
	outData->imgData = imgData;
	outData->xdim = xdim;
	outData->ydim = ydim;
	outData->colors = NIL;

	return(TRUE);
}
